/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package service;

import categorization.Advertisement;
import javax.ws.rs.Path;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.DELETE;
import javax.ws.rs.Produces;
import javax.ws.rs.Consumes;
import javax.ws.rs.QueryParam;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;
import javax.ws.rs.WebApplicationException;
import javax.persistence.NoResultException;
import javax.persistence.EntityManager;
import converter.AdvertisementConverter;
import javax.ejb.Stateless;
import org.netbeans.saas.google.GoogleMapService;
import org.netbeans.saas.RestResponse;

/**
 *
 * @author pico
 */
@Stateless
public class AdvertisementResource {

    @Context
    protected UriInfo uriInfo;
    protected EntityManager em;
    protected Integer id;

    /** Creates a new instance of AdvertisementResource */
    public AdvertisementResource() {
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public void setEm(EntityManager em) {
        this.em = em;
    }

    @GET
    @Produces("text/html")
    public String getGoogleMap() {
        // Drag and drop the getGoogleMap operation here

        try {
            Advertisement c = getEntity();
            String address = c.getLocation();
            java.lang.Integer zoom = 15;
            String iframe = "false";
            RestResponse result = GoogleMapService.getGoogleMap(address, zoom, iframe);
            return result.getDataAsString();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        return "";
    }

    /**
     * Get method for retrieving an instance of Advertisement identified by id in XML format.
     *
     * @param id identifier for the entity
     * @return an instance of AdvertisementConverter
     */
    @GET
    @Produces({"application/xml", "application/json"})
    public AdvertisementConverter get(@QueryParam("expandLevel")
            @DefaultValue("1") int expandLevel) {
        return new AdvertisementConverter(getEntity(), uriInfo.getAbsolutePath(), expandLevel);
    }

    /**
     * Put method for updating an instance of Advertisement identified by id using XML as the input format.
     *
     * @param id identifier for the entity
     * @param data an AdvertisementConverter entity that is deserialized from a XML stream
     */
    @PUT
    @Consumes({"application/xml", "application/json"})
    public void put(AdvertisementConverter data) {
        updateEntity(getEntity(), data.resolveEntity(em));
    }

    /**
     * Delete method for deleting an instance of Advertisement identified by id.
     *
     * @param id identifier for the entity
     */
    @DELETE
    public void delete() {
        deleteEntity(getEntity());
    }

    /**
     * Returns an instance of Advertisement identified by id.
     *
     * @param id identifier for the entity
     * @return an instance of Advertisement
     */
    protected Advertisement getEntity() {
        try {
            return (Advertisement) em.createQuery("SELECT e FROM Advertisement e where e.adid = :adid").setParameter("adid", id).getSingleResult();
        } catch (NoResultException ex) {
            throw new WebApplicationException(new Throwable("Resource for " + uriInfo.getAbsolutePath() + " does not exist."), 404);
        }
    }

    /**
     * Updates entity using data from newEntity.
     *
     * @param entity the entity to update
     * @param newEntity the entity containing the new data
     * @return the updated entity
     */
    private Advertisement updateEntity(Advertisement entity, Advertisement newEntity) {
        entity = em.merge(newEntity);
        return entity;
    }

    /**
     * Deletes the entity.
     *
     * @param entity the entity to deletle
     */
    private void deleteEntity(Advertisement entity) {
        em.remove(entity);
    }
}
