/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package service;

import categorization.Advertisement;
import java.util.Collection;
import javax.ws.rs.Path;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Produces;
import javax.ws.rs.Consumes;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;
import javax.persistence.EntityManager;
import converter.AdvertisementsConverter;
import converter.AdvertisementConverter;
import javax.persistence.PersistenceContext;
import javax.ejb.Stateless;

/**
 *
 * @author pico
 */

@Path("/advertisements/")
@Stateless
public class AdvertisementsResource {
    @javax.ejb.EJB
    private AdvertisementResource advertisementResource;
    @Context
    protected UriInfo uriInfo;
    @PersistenceContext(unitName = "adCategorizationPU")
    protected EntityManager em;
  
    /** Creates a new instance of AdvertisementsResource */
    public AdvertisementsResource() {
    }

    /**
     * Get method for retrieving a collection of Advertisement instance in XML format.
     *
     * @return an instance of AdvertisementsConverter
     */
    @GET
    @Produces({"application/xml", "application/json"})
    public AdvertisementsConverter get(@QueryParam("start")
                                       @DefaultValue("0")
    int start, @QueryParam("max")
               @DefaultValue("10")
    int max, @QueryParam("expandLevel")
             @DefaultValue("1")
    int expandLevel, @QueryParam("query")
                     @DefaultValue("SELECT e FROM Advertisement e")
    String query) {
        return new AdvertisementsConverter(getEntities(start, max, query), uriInfo.getAbsolutePath(), expandLevel);
    }

    /**
     * Post method for creating an instance of Advertisement using XML as the input format.
     *
     * @param data an AdvertisementConverter entity that is deserialized from an XML stream
     * @return an instance of AdvertisementConverter
     */
    @POST
    @Consumes({"application/xml", "application/json"})
    public Response post(AdvertisementConverter data) {
        Advertisement entity = data.resolveEntity(em);
        createEntity(data.resolveEntity(em));
        return Response.created(uriInfo.getAbsolutePath().resolve(entity.getAdid() + "/")).build();
    }

    /**
     * Returns a dynamic instance of AdvertisementResource used for entity navigation.
     *
     * @return an instance of AdvertisementResource
     */
    @Path("{adid}/")
    public AdvertisementResource getAdvertisementResource(@PathParam("adid")
    Integer id) {
        advertisementResource.setId(id);
        advertisementResource.setEm(em);
        return advertisementResource;
    }

    /**
     * Returns all the entities associated with this resource.
     *
     * @return a collection of Advertisement instances
     */
    protected Collection<Advertisement> getEntities(int start, int max, String query) {
        return em.createQuery(query).setFirstResult(start).setMaxResults(max).getResultList();
    }

    /**
     * Persist the given entity.
     *
     * @param entity the entity to persist
     */
    protected void createEntity(Advertisement entity) {
        entity.setAdid(null);
        em.persist(entity);
    }
}
